// Jacek Fedorynski <jfedor@jfedor.org>
// More info here: http://blog.jfedor.org/2014/06/morse-code-bluetooth-keyboard.html

#include <SendOnlySoftwareSerial.h>

const long UNIT_TIME = 75; // dit duration in milliseconds

const int KEY_PIN = 0;
const int LED_PIN = 1;
const int TX_PIN = 2;
const int BUZZER_PIN = 3;

SendOnlySoftwareSerial mySerial(TX_PIN);

const int ALPHABET_SIZE = 47;

char* codes[] = {
  ".-",
  "-...",
  "-.-.",
  "-..",
  ".",
  "..-.",
  "--.",
  "....",
  "..",
  ".---",
  "-.-",
  ".-..",
  "--",
  "-.",
  "---",
  ".--.",
  "--.-",
  ".-.",
  "...",
  "-",
  "..-",
  "...-",
  ".--",
  "-..-",
  "-.--",
  "--..",
  ".----",
  "..---",
  "...--",
  "....-",
  ".....",
  "-....",
  "--...",
  "---..",
  "----.",
  "-----",
  ".-.-.-",
  "--..--",
  "---...",
  "..--..",
  ".----.",
  "-....-",
  "-..-.",
  "-.--.",
  "-.--.-",
  ".-..-.",
  "-...-"
};

char letters[] = {
  'A',
  'B',
  'C',
  'D',
  'E',
  'F',
  'G',
  'H',
  'I',
  'J',
  'K',
  'L',
  'M',
  'N',
  'O',
  'P',
  'Q',
  'R',
  'S',
  'T',
  'U',
  'V',
  'W',
  'X',
  'Y',
  'Z',
  '1',
  '2',
  '3',
  '4',
  '5',
  '6',
  '7',
  '8',
  '9',
  '0',
  '.',
  ',',
  ':',
  '?',
  '\'',
  '-',
  '/',
  '(',
  ')',
  '"',
  '='
};

int prevstate = 0;
unsigned long prevtime;

void setup()                    
{
  pinMode(KEY_PIN, INPUT);
  digitalWrite(KEY_PIN, HIGH); // pull-up
  pinMode(LED_PIN, OUTPUT);
  digitalWrite(LED_PIN, LOW);
  pinMode(BUZZER_PIN, OUTPUT);
  digitalWrite(BUZZER_PIN, HIGH); // my buzzer makes a sound when the signal pin is driven low

  mySerial.begin(9600);

  prevtime = millis();
}

const int DITSLEN = 10;

char dits[DITSLEN];
int i = 0;

void emitLetter() {
  dits[i] = '\0';
  for (int j=0; j<ALPHABET_SIZE; j++) {
    if (strcmp(dits, codes[j]) == 0) {
      mySerial.write(letters[j]);
      break;
    }
  }
  i = 0;
}

void loop()                    
{
  int state = digitalRead(KEY_PIN) == LOW;
  digitalWrite(LED_PIN, state ? HIGH : LOW);
  digitalWrite(BUZZER_PIN, state ? LOW : HIGH); // my buzzer makes a sound when the signal pin is driven low
  
  unsigned long t = millis();
  unsigned long len = t-prevtime;

  if (prevstate != state) {
    if (prevstate) {
      if (len < 10) {
        // ignore
      } else if (len < UNIT_TIME*2) {
        // dit
        if (i < DITSLEN-1) {
          dits[i++] = '.';
        }
      } else {
        // dah
        if (i < DITSLEN - 1) {
          dits[i++] = '-';
        }
      }
    } else {
      if (len < UNIT_TIME*2) {
        // intra-letter gap
      } else if (len < UNIT_TIME*5.33) {
        // gap between letters
        emitLetter();
      }
    }
    
    prevstate = state;
    prevtime = t;
  } else {
    if (!prevstate && (i > 0) && (len >= UNIT_TIME*5.33)) {
      // gap between words
      emitLetter();
      mySerial.write(' ');
    }
  }
}
